@@ -20,10 +20,11 @@ mod fixture;
 
 mod markup;
 mod prime_caches;
-mod display;
+mod navigation_target;
 
 mod annotations;
 mod call_hierarchy;
+mod call_info;
 mod doc_links;
 mod highlight_related;
 mod expand_macro;
@@ -68,12 +69,12 @@ use ide_db::{
 };
 use syntax::SourceFile;
 
-use crate::display::ToNav;
+use crate::navigation_target::{ToNav, TryToNav};
 
 pub use crate::{
     annotations::{Annotation, AnnotationConfig, AnnotationKind},
     call_hierarchy::CallItem,
-    display::navigation_target::NavigationTarget,
+    call_info::CallInfo,
     expand_macro::ExpandedMacro,
     file_structure::{StructureNode, StructureNodeKind},
     folding_ranges::{Fold, FoldKind},
@@ -83,6 +84,7 @@ pub use crate::{
     join_lines::JoinLinesConfig,
     markup::Markup,
     move_item::Direction,
+    navigation_target::NavigationTarget,
     prime_caches::PrimeCachesProgress,
     references::ReferenceSearchResult,
     rename::RenameError,
@@ -106,7 +108,6 @@ pub use ide_db::{
         Cancelled, Change, CrateGraph, CrateId, Edition, FileId, FilePosition, FileRange,
         SourceRoot, SourceRootId,
     },
-    call_info::CallInfo,
     label::Label,
     line_index::{LineCol, LineColUtf16, LineIndex},
     search::{ReferenceCategory, SearchScope},
@@ -434,7 +435,7 @@ impl Analysis {
 
     /// Computes parameter information for the given call expression.
     pub fn call_info(&self, position: FilePosition) -> Cancellable<Option<CallInfo>> {
-        self.with_db(|db| ide_db::call_info::call_info(db, position))
+        self.with_db(|db| call_info::call_info(db, position))
     }
 
     /// Computes call hierarchy candidates for the given file position.
